@@ -3,7 +3,7 @@ schema.py - support for subSchemaSubEntry information
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: models.py,v 1.40 2011/06/29 08:57:04 stroeder Exp $
+\$Id: models.py,v 1.44 2011/07/22 17:19:08 stroeder Exp $
 """
 
 import UserDict,ldap.cidict
@@ -34,6 +34,19 @@ NOT_HUMAN_READABLE_LDAP_SYNTAXES = {
 class SchemaElement:
   """
   Base class for all schema element classes. Not used directly!
+
+  Arguments:
+  
+  schema_element_str
+    String which contains the schema element description to be parsed.
+
+  Class attributes:
+
+  schema_attribute
+    LDAP attribute type containing a certain schema element description
+  token_defaults
+    Dictionary internally used by the schema element parser
+    containing the defaults for certain schema description key-words
   """
   token_defaults = {
     'DESC':(None,),
@@ -87,17 +100,36 @@ class SchemaElement:
 
 class ObjectClass(SchemaElement):
   """
-  ObjectClassDescription = "(" whsp
-      numericoid whsp      ; ObjectClass identifier
-      [ "NAME" qdescrs ]
-      [ "DESC" qdstring ]
-      [ "OBSOLETE" whsp ]
-      [ "SUP" oids ]       ; Superior ObjectClasses
-      [ ( "ABSTRACT" / "STRUCTURAL" / "AUXILIARY" ) whsp ]
-                           ; default structural
-      [ "MUST" oids ]      ; AttributeTypes
-      [ "MAY" oids ]       ; AttributeTypes
-  whsp ")"
+  Arguments:
+  
+  schema_element_str
+    String containing an ObjectClassDescription
+
+  Class attributes:
+
+  oid
+    OID assigned to the object class
+  names
+    This list of strings contains all NAMEs of the object class
+  desc
+    This string contains description text (DESC) of the object class
+  obsolete
+    Integer flag (0 or 1) indicating whether the object class is marked
+    as OBSOLETE in the schema
+  must
+    This list of strings contains NAMEs or OIDs of all attributes
+    an entry of the object class must have
+  may
+    This list of strings contains NAMEs or OIDs of additional attributes
+    an entry of the object class may have
+  kind
+    Kind of an object class:
+    0 = ABSTRACT,
+    1 = STRUCTURAL,
+    2 = AUXILIARY
+  sup
+    This list of strings contains NAMEs or OIDs of object classes
+    this object class is derived from
   """
   schema_attribute = 'objectClasses'
   token_defaults = {
@@ -161,28 +193,47 @@ AttributeUsage = ldap.cidict.cidict({
 
 class AttributeType(SchemaElement):
   """
-      AttributeTypeDescription = "(" whsp
-            numericoid whsp              ; AttributeType identifier
-          [ "NAME" qdescrs ]             ; name used in AttributeType
-          [ "DESC" qdstring ]            ; description
-          [ "OBSOLETE" whsp ]
-          [ "SUP" woid ]                 ; derived from this other
-                                         ; AttributeType
-          [ "EQUALITY" woid              ; Matching Rule name
-          [ "ORDERING" woid              ; Matching Rule name
-          [ "SUBSTR" woid ]              ; Matching Rule name
-          [ "SYNTAX" whsp noidlen whsp ] ; see section 4.3
-          [ "SINGLE-VALUE" whsp ]        ; default multi-valued
-          [ "COLLECTIVE" whsp ]          ; default not collective
-          [ "NO-USER-MODIFICATION" whsp ]; default user modifiable
-          [ "USAGE" whsp AttributeUsage ]; default userApplications
-          whsp ")"
+  Arguments:
+  
+  schema_element_str
+    String containing an AttributeTypeDescription
 
-      AttributeUsage =
-          "userApplications"     /
-          "directoryOperation"   /
-          "distributedOperation" / ; DSA-shared
-          "dSAOperation"          ; DSA-specific, value depends on server
+  Class attributes:
+
+  oid
+    OID assigned to the attribute type
+  names
+    This list of strings contains all NAMEs of the attribute type
+  desc
+    This string contains description text (DESC) of the attribute type
+  obsolete
+    Integer flag (0 or 1) indicating whether the attribute type is marked
+    as OBSOLETE in the schema
+  single_value
+    Integer flag (0 or 1) indicating whether the attribute must
+    have only one value
+  syntax
+    String contains OID of the LDAP syntax assigned to the attribute type
+  no_user_mod
+    Integer flag (0 or 1) indicating whether the attribute is modifiable
+    by a client application
+  equality
+    String contains NAME or OID of the matching rule used for
+    checking whether attribute values are equal
+  substr
+    String contains NAME or OID of the matching rule used for
+    checking whether an attribute value contains another value
+  ordering
+    String contains NAME or OID of the matching rule used for
+    checking whether attribute values are lesser-equal than
+  usage
+    USAGE of an attribute type:
+    1 = directoryOperation,
+    2 = distributedOperation,
+    3 = dSAOperation
+  sup
+    This list of strings contains NAMEs or OIDs of attribute types
+    this attribute type is derived from
   """
   schema_attribute = 'attributeTypes'
   token_defaults = {
@@ -274,10 +325,15 @@ class AttributeType(SchemaElement):
 
 class LDAPSyntax(SchemaElement):
   """
-  SyntaxDescription = "(" whsp
-      numericoid whsp
-      [ "DESC" qdstring ]
-      whsp ")"
+  SyntaxDescription
+
+  oid
+    OID assigned to the LDAP syntax
+  desc
+    This string contains description text (DESC) of the LDAP syntax
+  not_human_readable
+    Integer flag (0 or 1) indicating whether the attribute type is marked
+    as not human-readable (X-NOT-HUMAN-READABLE)  
   """
   schema_attribute = 'ldapSyntaxes'
   token_defaults = {
@@ -304,13 +360,24 @@ class LDAPSyntax(SchemaElement):
 
 class MatchingRule(SchemaElement):
   """
-  MatchingRuleDescription = "(" whsp
-      numericoid whsp  ; MatchingRule identifier
-      [ "NAME" qdescrs ]
-      [ "DESC" qdstring ]
-      [ "OBSOLETE" whsp ]
-      "SYNTAX" numericoid
-  whsp ")"
+  Arguments:
+  
+  schema_element_str
+    String containing an MatchingRuleDescription
+
+  Class attributes:
+
+  oid
+    OID assigned to the matching rule
+  names
+    This list of strings contains all NAMEs of the matching rule
+  desc
+    This string contains description text (DESC) of the matching rule
+  obsolete
+    Integer flag (0 or 1) indicating whether the matching rule is marked
+    as OBSOLETE in the schema
+  syntax
+    String contains OID of the LDAP syntax this matching rule is usable with
   """
   schema_attribute = 'matchingRules'
   token_defaults = {
@@ -342,14 +409,25 @@ class MatchingRule(SchemaElement):
 
 class MatchingRuleUse(SchemaElement):
   """
-  MatchingRuleUseDescription = "(" whsp
-     numericoid 
-     [ space "NAME" space qdescrs ]
-     [ space "DESC" space qdstring ]
-     [ space "OBSOLETE" ]
-     space "APPLIES" space oids    ;  AttributeType identifiers
-     extensions
-     whsp ")" 
+  Arguments:
+  
+  schema_element_str
+    String containing an MatchingRuleUseDescription
+
+  Class attributes:
+
+  oid
+    OID of the accompanying matching rule
+  names
+    This list of strings contains all NAMEs of the matching rule
+  desc
+    This string contains description text (DESC) of the matching rule
+  obsolete
+    Integer flag (0 or 1) indicating whether the matching rule is marked
+    as OBSOLETE in the schema
+  applies
+    This list of strings contains NAMEs or OIDs of attribute types
+    for which this matching rule is used
   """
   schema_attribute = 'matchingRuleUse'
   token_defaults = {
@@ -381,16 +459,36 @@ class MatchingRuleUse(SchemaElement):
 
 class DITContentRule(SchemaElement):
   """
-  DITContentRuleDescription = LPAREN WSP
-      numericoid                 ; object identifer
-      [ SP "NAME" SP qdescrs ]   ; short names
-      [ SP "DESC" SP qdstring ]  ; description
-      [ SP "OBSOLETE" ]          ; not active
-      [ SP "AUX" SP oids ]       ; auxiliary object classes
-      [ SP "MUST" SP oids ]      ; attribute types
-      [ SP "MAY" SP oids ]       ; attribute types
-      [ SP "NOT" SP oids ]       ; attribute types
-      extensions WSP RPAREN      ; extensions
+  Arguments:
+  
+  schema_element_str
+    String containing an DITContentRuleDescription
+
+  Class attributes:
+
+  oid
+    OID of the accompanying structural object class
+  names
+    This list of strings contains all NAMEs of the DIT content rule
+  desc
+    This string contains description text (DESC) of the DIT content rule
+  obsolete
+    Integer flag (0 or 1) indicating whether the DIT content rule is marked
+    as OBSOLETE in the schema
+  aux
+    This list of strings contains NAMEs or OIDs of all auxiliary
+    object classes usable in an entry of the object class
+  must
+    This list of strings contains NAMEs or OIDs of all attributes
+    an entry of the object class must have which may extend the
+    list of required attributes of the object classes of an entry
+  may
+    This list of strings contains NAMEs or OIDs of additional attributes
+    an entry of the object class may have which may extend the
+    list of optional attributes of the object classes of an entry
+  nots
+    This list of strings contains NAMEs or OIDs of attributes which
+    may not be present in an entry of the object class
   """
   schema_attribute = 'dITContentRules'
   token_defaults = {
@@ -434,14 +532,27 @@ class DITContentRule(SchemaElement):
 
 class DITStructureRule(SchemaElement):
   """
-  DITStructureRuleDescription = LPAREN WSP
-      ruleid                     ; rule identifier
-      [ SP "NAME" SP qdescrs ]   ; short names
-      [ SP "DESC" SP qdstring ]  ; description
-      [ SP "OBSOLETE" ]          ; not active
-      SP "FORM" SP oid           ; NameForm
-      [ SP "SUP" ruleids ]       ; superior rules
-      extensions WSP RPAREN      ; extensions
+  Arguments:
+  
+  schema_element_str
+    String containing an DITStructureRuleDescription
+
+  Class attributes:
+
+  ruleid
+    rule ID of the DIT structure rule (only locally unique)
+  names
+    This list of strings contains all NAMEs of the DIT structure rule
+  desc
+    This string contains description text (DESC) of the DIT structure rule
+  obsolete
+    Integer flag (0 or 1) indicating whether the DIT content rule is marked
+    as OBSOLETE in the schema
+  form
+    List of strings with NAMEs or OIDs of associated name forms
+  sup
+    List of strings with NAMEs or OIDs of allowed structural object classes
+    of superior entries in the DIT
   """
   schema_attribute = 'dITStructureRules'
 
@@ -484,15 +595,33 @@ class DITStructureRule(SchemaElement):
 
 class NameForm(SchemaElement):
   """
-  NameFormDescription = LPAREN WSP
-      numericoid                 ; object identifer
-      [ SP "NAME" SP qdescrs ]   ; short names
-      [ SP "DESC" SP qdstring ]  ; description
-      [ SP "OBSOLETE" ]          ; not active
-      SP "OC" SP oid             ; structural object class
-      SP "MUST" SP oids          ; attribute types
-      [ SP "MAY" SP oids ]       ; attribute types
-      extensions WSP RPAREN      ; extensions
+  Arguments:
+  
+  schema_element_str
+    String containing an NameFormDescription
+
+  Class attributes:
+
+  oid
+    OID of the name form
+  names
+    This list of strings contains all NAMEs of the name form
+  desc
+    This string contains description text (DESC) of the name form
+  obsolete
+    Integer flag (0 or 1) indicating whether the name form is marked
+    as OBSOLETE in the schema
+  form
+    List of strings with NAMEs or OIDs of associated name forms
+  oc
+    String with NAME or OID of structural object classes this name form
+    is usable with
+  must
+    This list of strings contains NAMEs or OIDs of all attributes
+    an RDN must contain
+  may
+    This list of strings contains NAMEs or OIDs of additional attributes
+    an RDN may contain
   """
   schema_attribute = 'nameForms'
   token_defaults = {
